import { MouseCursor } from "../../interfaces/frontend"
import { count } from "./Gamefield"
import { borderCN, cornerCN, fieldIndex } from "@lib/utils/helpers"
import { CSSProperties, Dispatch, SetStateAction } from "react"

type TilesType = {
  key: number
  isGameTile: boolean
  className: string
  x: number
  y: number
}

function BorderTiles({
  props: { settingTarget, setMouseCursor },
}: {
  props: {
    settingTarget: (isGameTile: boolean, x: number, y: number) => void
    setMouseCursor: Dispatch<SetStateAction<MouseCursor>>
  }
}) {
  let tilesProperties: TilesType[] = []

  for (let y = 0; y < count + 2; y++) {
    for (let x = 0; x < count + 2; x++) {
      const key = fieldIndex(count, x, y)
      const cornerReslt = cornerCN(count, x, y)
      const borderType = cornerReslt ? cornerReslt : borderCN(count, x, y)
      const isGameTile = x > 0 && x < count + 1 && y > 0 && y < count + 1
      const classNames = ["border-tile"]
      if (borderType) classNames.push("edge", borderType)
      if (isGameTile) classNames.push("game-tile")
      const className = classNames.join(" ")
      tilesProperties.push({
        key,
        className,
        isGameTile,
        x: x + 1,
        y: y + 1,
      })
    }
  }

  return (
    <>
      {tilesProperties.map(({ key, className, isGameTile, x, y }) => {
        return (
          <div
            key={key}
            className={className}
            style={{ "--x": x, "--y": y } as CSSProperties}
            onClick={() => settingTarget(isGameTile, x, y)}
            onMouseEnter={() =>
              setMouseCursor({ x, y, shouldShow: isGameTile })
            }
          ></div>
        )
      })}
    </>
  )
}

export default BorderTiles
