import { Draw, Hit, MouseCursor, Target } from "../../interfaces/frontend"
// import Bluetooth from "./Bluetooth"
// import FogImages from "./FogImages"
import Labeling from "./Labeling"
import Ships from "./Ships"
import BorderTiles from "@components/Gamefield/BorderTiles"
import EventBar from "@components/Gamefield/EventBar"
import HitElems from "@components/Gamefield/HitElems"
import Targets from "@components/Gamefield/Targets"
import { useDraw } from "@hooks/useDraw"
import {
  hitReducer,
  initlialTarget,
  initlialTargetPreview,
  initlialMouseCursor,
  overlapsWithAnyBorder,
  isAlreadyHit,
  targetList,
} from "@lib/utils/helpers"
import { CSSProperties, useCallback } from "react"
import { useEffect, useReducer, useState } from "react"

export const count = 12

function Gamefield() {
  const [target, setTarget] = useState<Target>(initlialTarget)
  const [targetPreview, setTargetPreview] = useState<Target>(
    initlialTargetPreview
  )
  const [mouseCursor, setMouseCursor] =
    useState<MouseCursor>(initlialMouseCursor)
  const [hits, DispatchHits] = useReducer(hitReducer, [] as Hit[])
  const [mode, setMode] = useState(0)

  const settingTarget = useCallback(
    (isGameTile: boolean, x: number, y: number) => {
      const list = targetList(targetPreview, mode)
      if (
        !isGameTile ||
        !list.filter(({ x, y }) => !isAlreadyHit(x, y, hits)).length
      )
        return
      if (target.show && target.x == x && target.y == y) {
        DispatchHits({
          type: "fireMissile",
          payload: list.map(({ x, y }) => ({
            hit: isAlreadyHit(x, y, hits),
            x,
            y,
          })),
        })
        setTarget((t) => ({ ...t, show: false }))
      } else if (!overlapsWithAnyBorder(targetPreview, mode))
        setTarget({ show: true, x, y })
    },
    [hits, mode, target, targetPreview]
  )

  useEffect(() => {
    const { x, y, show } = target
    const { shouldShow, ...position } = mouseCursor
    if (!shouldShow || overlapsWithAnyBorder(position, mode))
      setTargetPreview((e) => ({ ...e, show: false }))
    else {
      setTargetPreview({
        ...position,
        show: !show || x !== position.x || y !== position.y,
      })
    }
  }, [mode, mouseCursor, target])

  const [color, setColor] = useState<string>("#f00")
  const [disable, setDisable] = useState(false)
  const { canvasRef, onMouseDown, clear } = useDraw(drawLine)

  function drawLine({ prevPoint, currentPoint, ctx }: Draw) {
    const { x: currX, y: currY } = currentPoint
    const lineColor = color
    const lineWidth = 5

    let startPoint = prevPoint ?? currentPoint
    ctx.beginPath()
    ctx.lineWidth = lineWidth
    ctx.strokeStyle = lineColor
    ctx.moveTo(startPoint.x, startPoint.y)
    ctx.lineTo(currX, currY)
    ctx.stroke()

    ctx.fillStyle = lineColor
    ctx.beginPath()
    ctx.arc(startPoint.x, startPoint.y, 2, 0, 2 * Math.PI)
    ctx.fill()
  }

  return (
    <div id="gamefield">
      {/* <Bluetooth /> */}
      <div
        id="game-frame"
        style={{ "--i": count } as CSSProperties}
        onMouseLeave={() =>
          setMouseCursor((e) => ({ ...e, shouldShow: false }))
        }
      >
        {/* Bordes */}
        <BorderTiles props={{ settingTarget, setMouseCursor }} />

        {/* Collumn lettes and row numbers */}
        <Labeling count={count} />

        {/* Ships */}
        <Ships />
        <HitElems hits={hits} />

        {/* Fog images */}
        {/* <FogImages /> */}

        <Targets props={{ target, targetPreview, mode, hits }} />
        {/* <span id='dev-debug' style={{gridArea: '1 / 12 / 1 / 15', backgroundColor: 'red', zIndex: 3} as CSSProperties}>Debug</span> */}
        <canvas
          style={
            {
              pointerEvents: !disable ? "auto" : "none",
            } as CSSProperties
          }
          ref={canvasRef}
          onMouseDown={onMouseDown}
          width="648"
          height="648"
        />
      </div>
      <button onClick={() => setDisable((e) => !e)}>toggle disable</button>
      <button onClick={clear}>Clear</button>
      <EventBar props={{ setMode, setTarget }} />
    </div>
  )
}

export default Gamefield
