import { GameSettings } from "@components/Lobby/SettingsFrame/Setting"
import { GamePropsSchema, PlayerSchema } from "@lib/zodSchemas"
import { PlayerN, User } from "@prisma/client"
import type { Server as HTTPServer } from "http"
import type { Socket as NetSocket } from "net"
import type { NextApiResponse } from "next"
import type {
  Server as IOServer,
  Server,
  Socket as SocketforServer,
} from "socket.io"
import type { Socket as SocketforClient } from "socket.io-client"

interface SocketServer extends HTTPServer {
  io?: IOServer
}

interface SocketWithIO extends NetSocket {
  server: SocketServer
}

export interface NextApiResponseWithSocket extends NextApiResponse {
  socket: SocketWithIO
}

export interface ServerToClientEvents {
  // noArg: () => void
  // basicEmit: (a: number, b: string, c: Buffer) => void
  // withAck: (d: string, ) => void
  gameSetting: (payload: GameSettings, hash: string, userId: string) => void
  playerEvent: (
    type: "join" | "leave" | "disconnect",
    payload: { player1?: PlayerSchema; player2?: PlayerSchema },
    hash: string,
    userId: string
  ) => void
  isReady: (
    payload: {
      index: PlayerN
      isReady: boolean
    },
    hash: string,
    userId: string
  ) => void
}

export interface ClientToServerEvents {
  update: (callback: (game: GamePropsSchema) => void) => void
  isReady: (isReady: boolean) => void
  ping: (count: number, callback: (count: number) => void) => void
  join: (withAck: (ack: boolean) => void) => void
  gameSetting: (payload: GameSettings, callback: (hash: string) => void) => void
  leave: (withAck: (ack: boolean) => void) => void
}

interface InterServerEvents {
  // ping: () => void
}

interface SocketData {
  user: User | null
  gameId: string | null
}

export type sServer = Server<
  ClientToServerEvents,
  ServerToClientEvents,
  InterServerEvents,
  SocketData
>
export type sSocket = SocketforServer<
  ClientToServerEvents,
  ServerToClientEvents,
  InterServerEvents,
  SocketData
>

export type cSocket = SocketforClient<
  ServerToClientEvents,
  ClientToServerEvents
>
